from __future__ import print_function

import inspect
import sys, os
from collections import namedtuple
import re
import PoserLibs.POSER_FAKE as poser
PARMS = namedtuple("PARMS", "Name args doc")
classnames = set()

sc = poser.Scene()

def debug(*args):
    print(*args, file=sys.stderr)

def analyze(obj2analyze):
    container = dict(
        classes=list(),
        builtins=list(),
        consts=list()
        )

    members = inspect.getmembers(obj2analyze)
    if members:
        for name, obj in members:
            if name.startswith("_"):
                continue
            doc = obj.__doc__.replace("  ","\n").split("\n") if obj.__doc__ else None
            if name.startswith("k"):
                container["consts"].append(PARMS(name, obj, ""))
            elif inspect.isclass(obj):
                container["classes"].append(PARMS(name, None, doc))
                classnames.add(name.strip().lower())
            elif inspect.isbuiltin(obj):
                container["builtins"].append(PARMS(name, None, doc))
    else:
        t = getattr(poser, obj2analyze.__class__.__name__ + "Type")
        if t:
            for name in dir(t):
                if name.startswith("_"):
                    continue
                obj = getattr(obj2analyze, name)
                doc = obj.__doc__.replace("  ","\n").split("\n") if obj.__doc__ else None
                container["builtins"].append(PARMS(name, None, doc))
        
    return container



def print_obj(container, tabs=0):
    if not container:
        debug(">>>>>>>>>EMPTY")
        return
    t = "\t" * tabs
    for entry in sorted(container["consts"]):
        if entry.doc:
            print(t + "# ".join(entry.doc))
        print("{}{} = {}".format(t, entry.Name, entry.args))
    print()

    for entry in sorted(container["classes"]):
        print("\n{}class {}:".format(t, entry.Name))
        if entry.doc:
            doc = ("\n" + t + "\t").join(entry.doc)
            print('{}"""\n{}\n{}"""'.format(t, doc, t))
        if entry.args is not None:
            print("{}\tdef __init__({}):".format(t, entry.args))
            print("{}\t\tpass".format(t))

        actor = geom = parm = figure = material = shadertree = node = None
        actor = poser.Scene().CurrentActor()
        figure = poser.Scene().CurrentFigure()
        if actor:
            geom = actor.Geometry()
            parm = actor.Parameters()[0]
            material = actor.Materials()[0]
            if material:
                shadertree = material.ShaderTree()
                if shadertree:
                    node = shadertree.Nodes()[0]
        try:
            if entry.Name == "SceneType":
                print_obj(analyze(poser.Scene()), tabs + 1)
            elif entry.Name == "ActorType" and actor:
                print_obj(analyze(actor), tabs + 1)
            elif entry.Name == "ParmType" and parm:
                print_obj(analyze(parm), tabs + 1)
            elif entry.Name == "GeomType" and geom:
                print_obj(analyze(geom), tabs + 1)
            elif entry.Name == "PolygonType" and geom:
                print_obj(analyze(geom.Polygons()[0]), tabs + 1)
            elif entry.Name == "TexPolygonType" and geom:
                print_obj(analyze(geom.TexPolygons()[0]), tabs + 1)
            elif entry.Name == "VertType" and geom:
                print_obj(analyze(geom.Vertices()[0]), tabs + 1)
            elif entry.Name == "TexVertType" and geom:
                print_obj(analyze(geom.Vertices()[0]), tabs + 1)
            elif entry.Name == "FigureType" and figure:
                print_obj(analyze(figure), tabs + 1)
            elif entry.Name == "MaterialType" and material:
                print_obj(analyze(material), tabs + 1)
            elif entry.Name == "ShaderTreeType" and shadertree:
                print_obj(analyze(shadertree), tabs + 1)
            elif entry.Name == "ShaderNodeType" and node:
                print_obj(analyze(node), tabs + 1)
            elif entry.Name == "ShaderNodeInputType" and node:
                print_obj(analyze(node.Inputs()[0]), tabs + 1)

            else:
                print("{}\tpass".format(t))
        except poser.error as err:
                debug(err)
                print("{}\tpass".format(t))

        print()


    for entry in sorted(container["builtins"]):
        args = entry.args or "*args"
        if tabs > 0:
            args = "self, {}".format(entry.args or "*args")
        print("\n{}def {}({}):".format(t, entry.Name, args))
        if entry.doc:
            doc = ("\n" + t + "\t").join(entry.doc)
            print('{}\t"""\n{}\t{}\n{}\t"""'.format(t, t, doc, t))

            res = re.match(r"\(?<([^>]+)>", entry.doc[-1])
            if res:
                return_type = res.groups(1)[0].strip()
                is_list = False
                if return_type.endswith("List"):
                    return_type = return_type.split()[0]
                    is_list = True

                return_str = ""
                #print(return_type, return_type.lower() in classnames, file=sys.stderr)
                if return_type.lower() in classnames:
                    return_str = return_type + "()"
                elif return_str == "NoneType":
                    return_str = None
                elif return_type == "IntType":
                    return_str = "int(1)"
                elif return_type == "FloatType":
                    return_str = "float(1.0)"
                elif return_type == "StringType":
                    return_str = "str({}())".format(entry.Name)
                elif return_type == "DictType":
                    return_str = 'dict()'

                if is_list:
                    return_str = "list({})".format(return_str)

                if return_str:
                    print("{}\treturn {}".format(t, return_str))
                else:
                    print("{}\tpass".format(t))
            else:
                print("{}\tpass".format(t))

temp_dir = poser.TempLocation()
fname = os.path.join(temp_dir, "POSER_FAKE.py")

_stdout = sys.stdout
try:
    with open(fname, "w") as fh:
        sys.stdout = fh
        print_obj(analyze(poser))
        sys.stdout = _stdout
except Exception as err:
    sys.stdout = _stdout
    print(err)

print("Written to:", fname)
